//
// Created by Tom on 08/03/2023.
//

#ifndef TYPON_DICT_HPP
#define TYPON_DICT_HPP

#include <unordered_map>
/*
template <typename K, typename V>
class PyDict : public std::unordered_map<K, V> {
public:
  PyDict(std::unordered_map<K, V> &&m)
      : std::unordered_map<K, V>(std::move(m)) {}

  PyDict(std::initializer_list<std::pair<const K, V>> m)
      : std::unordered_map<K, V>(m) {}

  operator std::unordered_map<K, V>() const {
    return std::unordered_map<K, V>(this->begin(), this->end());
  }

  operator std::unordered_map<K, V> &() {
    return *reinterpret_cast<std::unordered_map<K, V> *>(this);
  }

  std::size_t py_len() const { return this->size(); }

  bool py_contains(const K &k) const { return this->find(k) != this->end(); }

  class iterator {
  public:
    using value_type = std::pair<K, V>;
    using difference_type = std::ptrdiff_t;
    using pointer = std::pair<K, V> *;
    using reference = std::pair<K, V> &;
    using iterator_category = std::forward_iterator_tag;

    iterator(typename std::unordered_map<K, V>::iterator it) : _it(it) {}

    iterator &operator++() {
      _it++;
      return *this;
    }

    iterator operator++(int) {
      iterator tmp = *this;
      _it++;
      return tmp;
    }

    bool operator==(const iterator &rhs) const { return _it == rhs._it; }

    bool operator!=(const iterator &rhs) const { return _it != rhs._it; }

    const std::pair<K, V> &operator*() const { return *_it; }

    const std::pair<K, V> *operator->() const { return &*_it; }

  private:
    typename std::unordered_map<K, V>::iterator _it;
  };

  iterator py_iter() const { return this->begin(); }

  void py_print(std::ostream &s) const {
    s << '{';
    if (this->size() > 0) {
      print_to(this->begin()->first, s);
      s << ": ";
      print_to(this->begin()->second, s);
      for (auto it = ++this->begin(); it != this->end(); it++) {
        s << ", ";
        print_to(it->first, s);
        s << ": ";
        print_to(it->second, s);
      }
    }
    s << '}';
  }
};

template <typename K, typename V>
PyDict(std::initializer_list<std::pair<K, V>>) -> PyDict<K, V>;*/

template <typename K, typename V> class PyDict {
public:
  PyDict(std::shared_ptr<std::unordered_map<K, V>> &&m) : _m(std::move(m)) {}
  PyDict(std::unordered_map<K, V> &&m)
      : _m(std::move(
            std::make_shared<std::unordered_map<K, V>>(std::move(m)))) {}
  PyDict(std::initializer_list<std::pair<K, V>> &&m)
      : _m(std::make_shared<std::unordered_map<K, V>>(std::move(m))) {}

  PyDict() : _m(std::make_shared<std::unordered_map<K, V>>()) {}

  auto begin() const { return _m->begin(); }
  auto end() const { return _m->end(); }

  auto py_contains(const K &k) const { return _m->find(k) != _m->end(); }

  constexpr const V &operator[](const K &k) const { return (*_m)[k]; }
  constexpr V &operator[](const K &k) { return (*_m)[k]; }

  size_t py_len() const { return _m->size(); }

  void py_repr(std::ostream &s) const {
    s << '{';
    if (_m->size() > 0) {
      repr_to(_m->begin()->first, s);
      s << ": ";
      repr_to(_m->begin()->second, s);
      for (auto it = ++_m->begin(); it != _m->end(); it++) {
        s << ", ";
        repr_to(it->first, s);
        s << ": ";
        repr_to(it->second, s);
      }
    }
    s << '}';
  }

  void py_print(std::ostream &s) const { py_repr(s); }

private:
  std::shared_ptr<std::unordered_map<K, V>> _m;
};

#endif // TYPON_DICT_HPP
