//
// Created by Tom on 08/03/2023.
//

#ifndef TYPON_LIST_HPP
#define TYPON_LIST_HPP

#include <algorithm>
#include <ostream>
#include <vector>

template <typename T> class PyList {
public:
  PyList(std::shared_ptr<std::vector<T>> &&v) : _v(std::move(v)) {}
  PyList(std::vector<T> &&v)
      : _v(std::move(std::make_shared<std::vector<T>>(std::move(v)))) {}

  PyList(std::initializer_list<T> &&v)
      : _v(std::make_shared<std::vector<T>>(std::move(v))) {}

  PyList() : _v(std::make_shared<std::vector<T>>()) {}

  auto begin() const { return _v->begin(); }

  auto end() const { return _v->end(); }

  auto append(const T &x) { _v->push_back(x); }

  auto py_contains(const T &x) const {
    return std::find(_v->begin(), _v->end(), x) != _v->end();
  }

  /*operator std::vector<T>() const {
    return std::vector<T>(this->begin(), this->end());
  }

  operator std::vector<T> &() {
    return *reinterpret_cast<std::vector<T> *>(this);
  }*/

  constexpr const T &operator[](size_t i) const { return _v->operator[](i); }
  constexpr T &operator[](size_t i) { return _v->operator[](i); }

  size_t py_len() const { return _v->size(); }

  void py_repr(std::ostream &s) const {
    s << '[';
    if (_v->size() > 0) {
      repr_to(_v->operator[](0), s);
      for (size_t i = 1; i < _v->size(); i++) {
        s << ", ";
        repr_to(_v->operator[](i), s);
      }
    }
    s << ']';
  }

  void py_print(std::ostream &s) const { py_repr(s); }

  PyList<T> operator+(const PyList<T> &other) const {
    std::vector<T> v;
    v.reserve(_v->size() + other._v->size());
    v.insert(v.end(), _v->begin(), _v->end());
    v.insert(v.end(), other._v->begin(), other._v->end());
    return PyList<T>(std::move(v));
  }

  PyList<T> operator*(size_t n) const {
    PyList<T> v{};
    v._v->reserve(this->_v->size() * n);
    for (size_t i = 0; i < n; i++) {
      v._v->insert(v._v->end(), this->_v->begin(), this->_v->end());
    }
    return v;
  }

private:
  std::shared_ptr<std::vector<T>> _v;
};

template <typename T> PyList<T> list(std::initializer_list<T> &&v) {
  return PyList<T>(std::move(v));
}

#endif // TYPON_LIST_HPP
