//
// Created by Tom on 08/03/2023.
//

#ifndef TYPON_STR_HPP
#define TYPON_STR_HPP

#include <sstream>
#include <string>

using namespace std::literals;

#include "bytes.hpp"
#include "print.hpp"
// #include <format>
#include <fmt/format.h>

class PyStr : public std::string {
public:
  PyStr() : std::string() {}
  PyStr(const char *s) : std::string(s) {}
  PyStr(const std::string &s) : std::string(s) {}
  PyStr(std::string &&s) : std::string(std::move(s)) {}
  PyStr(size_t count, char ch) : std::string(count, ch) {}
  constexpr PyStr(const char *s, size_t count) : std::string(s, count) {}

  template <class InputIterator>
  PyStr(InputIterator first, InputIterator last) : std::string(first, last) {}

  PyBytes encode(const std::string &encoding = "utf-8") const {
    return PyBytes(this->begin(), this->end());
  }

  template <typename... T> PyStr format(T &&...args) const {
    return PyStr(fmt::format(fmt::runtime(*this), std::forward<T>(args)...));
  }

  bool startswith(const std::string &s) const { return this->starts_with(s); }
};

inline constexpr PyStr operator""_ps(const char *s, size_t len) noexcept {
  return PyStr(s, len);
}

PyStr PyBytes::decode(const std::string &encoding) const {
  return PyStr(this->begin(), this->end());
}

template <typename T> PyStr str(const T &x) {
  std::stringstream s;
  print_to(x, s);
  return s.str();
}

template <typename T> PyStr repr(const T &x) {
  std::stringstream s;
  repr_to(x, s);
  return s.str();
}

template <> struct std::hash<PyStr> {
  std::size_t operator()(const PyStr &s) const noexcept {
    return std::hash<std::string>()(s);
  }
};

#endif // TYPON_STR_HPP
