//
// Created by Tom on 13/06/2023.
//

#ifndef TYPON_SOCKET_HPP
#define TYPON_SOCKET_HPP

#include "builtins.hpp"
#include <netinet/in.h>
#include <tuple>

namespace py_socket {
struct socket_t {
#undef SOCK_STREAM
#undef AF_INET6
#undef SOL_SOCKET
#undef SO_REUSEADDR
  static constexpr int SOCK_STREAM = 1;
  static constexpr int AF_INET6 = 10;
  static constexpr int SOL_SOCKET = 1;
  static constexpr int SO_REUSEADDR = 2;

  struct {
    struct type {
      METHOD(typon::Task<std::tuple<type COMMA() std::string>>, accept, (), {
        int connfd = co_await typon::io::accept(self->fd, NULL, NULL);
        if (connfd < 0) {
          system_error(-connfd, "accept()");
        }
        co_return std::make_tuple(type(connfd), std::string("")); // TODO
      })

      METHOD(typon::Task<void>, close, (),
             { co_await typon::io::close(self->fd); })

      METHOD(void, listen, (int backlog), {
        if (::listen(self->fd, backlog) < 0) {
          self->close();
          system_error(errno, "listen()");
        }
      })

      METHOD(void, setsockopt, (int level, int optname, int optval), {
        if (::setsockopt(self->fd, level, optname, &optval, sizeof(int)) < 0) {
          system_error(errno, "setsockopt()");
        }
      })

      METHOD(void, bind, (std::tuple<std::string COMMA() int> address), {
        auto [host, port] = address;
        sockaddr_in6 addr;
        std::memset(&addr, 0, sizeof(addr));
        addr.sin6_family = AF_INET6;
        addr.sin6_port = htons(port);
        addr.sin6_addr = in6addr_any;
        if (::bind(self->fd, (const sockaddr *)&addr, sizeof(addr)) < 0) {
          system_error(errno, "bind()");
        }
      })

      METHOD(typon::Task<PyBytes>, recv, (int bufsize), {
        PyBytes buf(bufsize, '\0');
        co_await typon::io::recv(self->fd, buf.data(), buf.size(), 0);
        co_return std::move(buf);
      })

      METHOD(typon::Task<void>, send, (PyBytes data), {
        if (int sbytes = co_await typon::io::send(self->fd, data, 0);
            sbytes < 0) {
          co_await self->close();
          system_error(-sbytes, "send()");
        }
      })

      type(int fd = -1) : fd(fd) {}

      type(const type &other)
          : fd(other.fd), accept(this), close(this), listen(this),
            setsockopt(this), bind(this), recv(this), send(this) {}

      int fd;
    };

    type operator()(int family, int type_) {
      if (int fd = ::socket(family, type_, 0); fd >= 0) {
        return type(fd);
      } else {
        system_error(errno, "socket()");
      }
    }
  } socket;
} all;

auto &get_all() { return all; }
} // namespace py_socket

namespace typon {
using PySocket = decltype(py_socket::all.socket)::type;
}

#endif // TYPON_SOCKET_HPP
