# coding: utf-8
import argparse
from os import system, environ
from pathlib import Path

from transpiler import transpile
from transpiler.format import format_code

# load .env file
from dotenv import load_dotenv

load_dotenv()

parser = argparse.ArgumentParser()
parser.add_argument("-c", "--compile", action="store_true")
args = parser.parse_args()

def run_tests():


    for path in Path('tests').glob('*.py'):
        print(path.name)
        if path.name.startswith('_'):
            print("Skipping")
            continue
        with open(path, "r", encoding="utf-8") as f:
            code = f.read()
            execute = "# norun" not in code
            res = format_code(transpile(code))
        #print(res)
        name_cpp = path.with_suffix('.cpp')
        with open(name_cpp, "w", encoding="utf-8") as fcpp:
            fcpp.write(res)
        print(".cpp generated")
        if args.compile:
            continue
        if not execute:
            print("Not executing", path.name)
            continue
        name_bin = path.with_suffix('').as_posix()
        commands = [
           f"bash -c 'PYTHONPATH=stdlib python3 ./{path.as_posix()}'",
        ]
        if alt := environ.get("ALT_RUNNER"):
            commands.append(alt.format(name_bin=name_bin, name_cpp_posix=name_cpp.as_posix()))
        else:
            print("no ALT_RUNNER")
        for cmd in commands:
            if system(cmd) != 0:
                print(f"Error running command: {cmd}")
                break


if __name__ == "__main__":
    run_tests()
