# coding: utf-8
import ast

from transpiler.phases.utils import PlainBlock


def process(items: list[ast.withitem], body: list[ast.stmt]) -> PlainBlock:
    first, *rest = items
    val, name = first.context_expr, first.optional_vars
    cm_name = ast.Name(id=f"cm_{hash(first)}")
    res = [
        ast.Assign(targets=[cm_name], value=val)
    ]
    enter_call = ast.Call(func=ast.Attribute(value=cm_name, attr="__enter__"), args=[], keywords=[])
    if name:
        res.append(ast.Assign(targets=[name], value=enter_call))
    else:
        res.append(ast.Expr(value=enter_call))
    if rest:
        res.append(process(rest, body))
    else:
        res.append(PlainBlock(body))
    res.append(ast.Expr(value=ast.Call(func=ast.Attribute(value=cm_name, attr="__exit__"), args=[], keywords=[])))
    return PlainBlock(res)


class DesugarWith(ast.NodeTransformer):
    def visit_With(self, node: ast.With):
        return process(node.items, node.body)