# coding: utf-8
import ast

SYMBOLS = {
    ast.Eq: "==",
    ast.NotEq: '!=',
    ast.Pass: '/* pass */',
    ast.Mult: '*',
    ast.Add: '+',
    ast.Sub: '-',
    ast.Div: '/',
    ast.FloorDiv: '/',  # TODO
    ast.Mod: '%',
    ast.Lt: '<',
    ast.Gt: '>',
    ast.GtE: '>=',
    ast.LtE: '<=',
    ast.LShift: '<<',
    ast.RShift: '>>',
    ast.BitXor: '^',
    ast.BitOr: '|',
    ast.BitAnd: '&',
    ast.Not: '!',
    ast.IsNot: '!=',
    ast.USub: '-',
    ast.And: '&&',
    ast.Or: '||'
}
"""Mapping of Python AST nodes to C++ symbols."""

PRECEDENCE = [
    ("()", "[]", ".",),
    ("unary", "co_await"),
    ("*", "/", "%",),
    ("+", "-"),
    ("<<", ">>"),
    ("<", "<=", ">", ">="),
    ("==", "!="),
    ("&",),
    ("^",),
    ("|",),
    ("&&",),
    ("||",),
    ("?:", "co_yield"),
    (",",)
]
"""Precedence of C++ operators."""

PRECEDENCE_LEVELS = {op: i for i, ops in enumerate(PRECEDENCE) for op in ops}
"""Mapping of C++ operators to their precedence level."""

MAPPINGS = {
    "True": "true",
    "False": "false",
    "None": "nullptr"
}
"""Mapping of Python builtin constants to C++ equivalents."""
