# coding: utf-8
import ast
from typing import Iterable

from dataclasses import dataclass, field
from transpiler.phases.typing.scope import Scope
from transpiler.phases.emit_cpp import CoroutineMode, FunctionEmissionKind, NodeVisitor
from transpiler.phases.emit_cpp.block import BlockVisitor
from transpiler.phases.emit_cpp.class_ import ClassVisitor
from transpiler.phases.emit_cpp.function import FunctionVisitor
from transpiler.utils import compare_ast


# noinspection PyPep8Naming
@dataclass
class ModuleVisitor(BlockVisitor):
    includes: list[str] = field(default_factory=list)
    def visit_Import(self, node: ast.Import) -> Iterable[str]:
        for alias in node.names:
            if alias.name in ("typon", "typing"):
                yield ""
            else:
                yield from self.import_module(alias.name)
                yield f'auto& {alias.asname or alias.name} = py_{alias.name}::get_all();'

    def import_module(self, name: str) -> Iterable[str]:
        self.includes.append(f'#include <python/{name}.hpp>')
        yield ""

    def visit_ImportFrom(self, node: ast.ImportFrom) -> Iterable[str]:
        if node.module in ("typon", "typing"):
            yield ""
        else:
            yield from self.import_module(node.module)
            for alias in node.names:
                yield f"auto& {alias.asname or alias.name} = py_{node.module}::get_all().{alias.name};"

    def visit_Expr(self, node: ast.Expr) -> Iterable[str]:
        if isinstance(node.value, ast.Str):
            if "\n" in node.value.s:
                yield f"/*{node.value.s}*/"
            else:
                yield f"//{node.value.s}"
        else:
            raise NotImplementedError(node)

    def visit_ClassDef(self, node: ast.ClassDef) -> Iterable[str]:
        yield from ClassVisitor().visit(node)

    def visit_FunctionDef(self, node: ast.FunctionDef) -> Iterable[str]:
        yield from super().visit_free_func(node, FunctionEmissionKind.DECLARATION)


@dataclass
class ModuleVisitor2(NodeVisitor):
    scope: Scope
    def visit_FunctionDef(self, node: ast.FunctionDef) -> Iterable[str]:
        yield from BlockVisitor(self.scope).visit_free_func(node, FunctionEmissionKind.DEFINITION)

    def visit_AST(self, node: ast.AST) -> Iterable[str]:
        yield ""
        pass
