import ast
from dataclasses import dataclass, field
from typing import Dict, Optional

from transpiler.phases.typing.annotations import TypeAnnotationVisitor
from transpiler.phases.typing.scope import Scope, ScopeKind, VarDecl
from transpiler.phases.typing.types import BaseType, TypeVariable, TY_NONE, TypeType
from transpiler.phases.utils import NodeVisitorSeq

PRELUDE = Scope.make_global()

@dataclass
class ScoperVisitor(NodeVisitorSeq):
    scope: Scope = field(default_factory=lambda: PRELUDE.child(ScopeKind.GLOBAL))
    root_decls: Dict[str, VarDecl] = field(default_factory=dict)
    cur_class: Optional[TypeType] = None

    def anno(self) -> "TypeAnnotationVisitor":
        return TypeAnnotationVisitor(self.scope, self.cur_class)

    def visit_annotation(self, expr: Optional[ast.expr]) -> BaseType:
        res = self.anno().visit(expr) if expr else TypeVariable()
        assert not isinstance(res, TypeType)
        return res

    def visit_block(self, block: list[ast.AST]):
        from transpiler.phases.typing.block import ScoperBlockVisitor
        self.fdecls = []
        for b in block:
            self.visit(b)
        for node, rtype in self.fdecls:
            for b in node.body:
                decls = {}
                visitor = ScoperBlockVisitor(node.inner_scope, decls)
                visitor.visit(b)
                b.decls = decls
            if not node.inner_scope.has_return:
                rtype.unify(TY_NONE)

