import ast

from dataclasses import dataclass

from transpiler.utils import UnsupportedNodeError


class NodeVisitorSeq:
    def visit(self, node):
        """Visit a node."""
        if type(node) == list:
            for n in node:
                self.visit(n)
        else:
            for parent in node.__class__.__mro__:
                if visitor := getattr(self, 'visit_' + parent.__name__, None):
                    try:
                        return visitor(node)
                    except Exception as e:
                        raise
                        #raise IncompatibleTypesError(f"{e} in `{ast.unparse(node)}`")
            else:
                self.missing_impl(node)

    def missing_impl(self, node):
        try:
            raise UnsupportedNodeError(str(node))
        except Exception as e:
            raise NotImplementedError(type(node))

@dataclass
class PlainBlock(ast.stmt):
    body: list[ast.stmt]